"""Translation/Localization functions.

Provides ``gettext`` translation functions via an app's ``pylons.translator``
and get/set_lang for changing the language translated to."""
import os
from gettext import NullTranslations, GNUTranslations
from pkg_resources import resource_exists, resource_stream
import pylons

class LanguageError(Exception):
    """Exception raised when a problem occurs with changing languages"""
    pass

def gettext_noop(value):
    """Mark a string for translation without translating it. Returns value.

    Used for global strings, e.g.:

    .. code-block:: Python

        foo = N_('Hello')

        class Bar:
            def __init__(self):
                self.local_foo = _(foo)

        h.set_lang('fr')
        assert Bar().local_foo == 'Bonjour'
        h.set_lang('es')
        assert Bar().local_foo == 'Hola'
        assert foo == 'Hello'
    """
    return value
N_ = gettext_noop

def gettext(value):
    """Mark a string for translation. Returns the localized string of value.
    
    Mark a string to be localized as follows:
    
    .. code-block:: Python
    
        gettext('This should be in lots of languages')
    """
    return pylons.translator.gettext(value)

def ugettext(value):
    """Mark a string for translation. Returns the localized unicode string of
    value.
    
    Mark a string to be localized as follows:
    
    .. code-block:: Python
    
        _('This should be in lots of languages')
    """
    return pylons.translator.ugettext(value)
_ = ugettext

def ngettext(singular, plural, n):
    """Mark a string for translation. Returns the localized string of the
    pluralized value.

    This does a plural-forms lookup of a message id. ``singular`` is used as
    the message id for purposes of lookup in the catalog, while ``n`` is used
    to determine which plural form to use. The returned message is a string.
    
    Mark a string to be localized as follows:
    
    .. code-block:: Python
    
        ngettext('There is %(num)d file here', 'There are %(num)d files here',
                 n) % {'num': n}
    """
    return pylons.translator.ngettext(singular, plural, n)

def ungettext(singular, plural, n):
    """Mark a string for translation. Returns the localized unicode string of
    the pluralized value.

    This does a plural-forms lookup of a message id. ``singular`` is used as
    the message id for purposes of lookup in the catalog, while ``n`` is used
    to determine which plural form to use. The returned message is a Unicode
    string.
    
    Mark a string to be localized as follows:
    
    .. code-block:: Python
    
        ungettext('There is %(num)d file here', 'There are %(num)d files here',
                  n) % {'num': n}
    """
    return pylons.translator.ungettext(singular, plural, n)

def set_lang(lang):
    """Set the i18n language used"""
    registry = pylons.request.environ['paste.registry']
    if lang is None:
        registry.replace(pylons.translator, NullTranslations())
    else:
        import pylons.util as util
        project_name = util.config_get('package')
        catalog_path = os.path.join('i18n', lang, 'LC_MESSAGES')
        if not resource_exists(project_name, catalog_path):
            raise LanguageError('Language catalog %s not found' % \
                                os.path.join(project_name, catalog_path))
        translator = egg_translation(project_name, lang=catalog_path)
        translator.pylons_lang = lang
        registry.replace(pylons.translator, translator)

def get_lang():
    """Return the current i18n language used"""
    return getattr(pylons.translator, 'pylons_lang', None)

def egg_translation(domain, lang):
    """Return a gettext Translations object for the specified domain and
    language.
    
    Like gettext.translation, but lacks its extensive checks and supports
    loading .mo files from inside of eggs."""
    class_ = GNUTranslations
    return class_(resource_stream(domain, os.path.join(lang, '%s.mo' % domain)))

__all__ = ['gettext_noop', 'N_', 'gettext', 'ugettext', '_', 'ngettext',
           'ungettext', 'set_lang', 'get_lang']



"""Legacy functionality for pre Pylons 0.9.3 projects
"""
import types
import sys
import warnings

import pylons

def load_h(package_name):
    """
    This is a legacy test for pre-0.9.3 projects to continue using the old
    style Helper imports. The proper style is to pass the helpers module ref
    to the PylonsApp during initialization.
    """
    __import__(package_name + '.lib.base')
    their_h = getattr(sys.modules[package_name + '.lib.base'], 'h', None)
    if isinstance(their_h, types.ModuleType):
        # lib.base.h is a module (and thus not pylons.h) -- assume lib.base uses
        # new style (self contained) helpers via:
        # import ${package}.lib.helpers as h
        return their_h

    # Assume lib.base.h is a StackedObjectProxy -- lib.base is using pre 0.9.2
    # style helpers via:
    # from pylons import h
    helpers_name = package_name + '.lib.helpers'
    __import__(helpers_name) 
    helpers_module = sys.modules[helpers_name]

    # Pre 0.9.2 lib.helpers did not import the pylons helper functions, manually
    # add them. Don't overwrite user functions (allowing pylons helpers to be
    # overridden)
    for func_name, func in {'_': pylons.i18n._, 'log': pylons.helpers.log,
                            'set_lang': pylons.i18n.set_lang,
                            'get_lang': pylons.i18n.get_lang}.iteritems():
        if not hasattr(helpers_module, func_name):
            setattr(helpers_module, func_name, func)

    return sys.modules[helpers_name]

__all__ = ['load_h']




"""Configuration setup for Myghty, and Paste error middleware

This module supplies pylons_config which handles setting up defaults
for Myghty, Paste errorware, and prefixing Routes if necessary.
"""
import os
import re
from paste.deploy.converters import asbool
import pylons.templating

class Config(object):
    """Pylons configuration object
    
    The Pylons configuration object is a per-application instance object
    that retains the information regarding the global and app conf's as
    well as per-application instance specific data such as the mapper, the
    paths for this instance, and the myghty configuration.
    
    The Config object is available in your application as a Pylons global
    ``pylons_config`` under the ``g`` object. There's several useful
    attributes of the config object most people will be interested in:
    
    ``myghty``
        The myghty configuration dict that was used to initialize Myghty
    ``template_options``
        Full dict of template options that any TG compatible plugin should
        be able to parse. Comes with basic config needed for Myghty, Kid,
        and Mako.
    ``map``
        Mapper object used for Routing. Yes, it is possible to add routes
        after your application has started running.
    ``paths``
        An array of absolute paths that were defined in the applications
        ``config/environment.py`` module.
    ``global_conf``
        Global configuration passed in from Paste, this corresponds to the
        DEFAULTS section in the config file.
    ``app_conf``
        Application specific configuration directives, passed in via Paste
        from the app section of the config file.
    ``environ_config``
        Dict of environ keys for where in the environ to pickup various
        objects for registering with Pylons. If these are present then
        PylonsApp will use them from environ rather than using default
        middleware from Beaker. Valid keys are: ``session, cache``
    ``template_engines``
        List of template engines to configure. The first one in the list will
        be configured as the default template engine. Each item in the list is
        a dict indicating how to configure the template engine with keys:
        ``engine``, ``template_root``, ``template_options``, and ``alias``
    ``default_charset``
        The default character encoding specified to the browser via the
        'charset' parameter of the HTTP response's Content-Type header.
    ``strict_c``
        Whether or not the ``c`` object should throw an attribute error when
        access is attempted to an attribute that doesn't exist.
    """
    def __init__(self, tmpl_options, map, paths, environ_config=None, 
        default_charset='UTF-8', strict_c=False):
        if environ_config is None:
            environ_config = {}
        self.myghty = tmpl_options
        self.template_options = tmpl_options
        self.map = map
        self.paths = paths
        self.environ_config = environ_config
        self.default_charset = default_charset
        self.strict_c = strict_c
        if 'output_encoding' not in tmpl_options:
            tmpl_options['output_encoding'] = default_charset
        self.global_conf = {}
        self.app_conf = {}
        self.template_engines = []
        self._clean_tmpl_options()
    
    def _clean_tmpl_options(self):
        """Prase the template options, set self.myghty to have myghty options
        with no prefix and template_options to be a proper set of options for 
        any template engine"""
        oldopts = self.myghty
        self.template_options = {}
        self.myghty = {}
        for k, v in oldopts.iteritems():
            if not re.match(r'\w+\.\w*', k):
                self.myghty[k] = v
            elif k.startswith('myghty.'):
                self.myghty[k[7:]] = v
            else:
                self.template_options[k] = v
    
    def add_template_engine(self, engine, root, options=None, alias=None):
        """Add additional template engines for configuration on Pylons WSGI init
        
        ``engine``
            The name of the template engine
        
        ``root``
            Template root for the engine
        
        ``options``
            Dict of additional options used during engine initialization, if
            not provided, default to using the template_options dict.
        
        ``alias``
            Name engine should respond to when actually used. This allows for
            multiple configurations of the same engine and lets you alias the
            additional ones to other names.
        
        Example of Kid addition:
        
        .. code-block:: Python
            
            # In yourproj/middleware.py
            # ...
            config.init_app(global_conf, app_conf, package='yourproj')

            # Load additional template engines
            kidopts = {'kid.assume_encoding':'utf-8', 'kid.encoding':'utf-8'}
            config.add_template_engine('kid', 'yourproj.kidtemplates', kidopts)
        
        Example of changing the default template engine:
        
        .. code-block:: Python

            # In yourproj/middleware.py
            # ...
            config.init_app(global_conf, app_conf, package='yourproj')
            
            # Remove existing template engine
            old_default = config.template_engines.pop()
            
            # Load additional template engines
            kidopts = {'kid.assume_encoding':'utf-8', 'kid.encoding':'utf-8'}
            config.add_template_engine('kid', 'yourproj.kidtemplates', kidopts)
            
            # Add old default as additional engine
            config.template_engines.append(old_default)
        """
        if not options:
            options = self.template_options
        config = dict(engine=engine, template_root=root, 
            template_options=options, alias=alias)
        self.template_engines.append(config)
    
    def init_app(self, global_conf, app_conf, package, template_engine='pylonsmyghty'):
        """Initialize configuration for the application
        
        ``global_config``
            Several options are expected to be set for a Pylons web application.
            They will be loaded from the global_config which has the main Paste
            options. If ``debug`` is not enabled as a global config option, the
            following option *must* be set:
            
            * error_to - The email address to send the debug error to
            
            The optional config options in this case are:
            
            * smtp_server - The SMTP server to use, defaults to 'localhost'
            * error_log - A logfile to write the error to
            * error_subject_prefix - The prefix of the error email subject
            * from_address - Whom the error email should be from
        ``app_conf``
            Defaults supplied via the [app:main] section from the Paste
            config file. ``load_config`` only cares about whether a 'prefix'
            option is set, if so it will update Routes to ensure URL's take
            that into account.
        ``package``
            The name of the application package, to be stored in the app_conf.
        ``template_engine``
            Declare the default template engine to setup. Choices are kid, genshi,
            mako, and pylonsmyghty (the default custom Pylons plugin).
        """
        self.global_conf = global_conf
        self.app_conf = app_conf
        self.package = package
        
        app_conf['package'] = package
        
        # Setup the prefix to override the routes if necessary.
        prefix = app_conf.get('prefix')
        if not prefix:
            prefix = global_conf.get('prefix')
        if prefix:
            self.map.prefix = app_conf['prefix']
            self.map._created_regs = False
        
        errorware = {}
        # Load the errorware configuration from the Paste configuration file
        # These all have defaults, and emails are only sent if configured and
        # if this application is running in production mode
        errorware['debug'] = asbool(global_conf.get('debug'))
        if not errorware['debug']:
            errorware['debug'] = False
            errorware['error_email'] = global_conf.get('email_to')
            errorware['error_log'] = global_conf.get('error_log', None)
            errorware['smtp_server'] = global_conf.get('smtp_server', 
                'localhost')
            errorware['error_subject_prefix'] = global_conf.get(
                'error_subject_prefix', 'WebApp Error: ')
            errorware['from_address'] = global_conf.get('from_address', 
                global_conf.get('error_email_from', 'pylons@yourapp.com'))
            errorware['error_message'] = global_conf.get('error_message', 
                'An internal server error occurred')
        
        # Standard Pylons configuration directives for Myghty
        myghty_defaults = {}
        
        # Raise a complete error for the error middleware to catch
        myghty_defaults['raise_error'] = True
        myghty_defaults['component_root'] = [{os.path.basename(path): path} \
            for path in self.paths['templates']]
                    
        # Merge in the user-supplied Myghty values
        myghty_defaults.update(self.myghty)
        
        # Merge additional globals
        myghty_defaults.setdefault('allow_globals',
                                   []).extend(pylons.templating.PYLONS_VARS)
        
        self.myghty = myghty_defaults
        myghty_template_options = {}
        if 'myghty_data_dir' in app_conf:
            myghty_defaults['data_dir'] = app_conf['myghty_data_dir']
        elif 'cache_dir' in app_conf:
            myghty_defaults['data_dir'] = os.path.join(app_conf['cache_dir'], 
                'templates')
        
        # Copy Myghty defaults and options into template options
        for k, v in self.myghty.iteritems():
            myghty_template_options['myghty.'+k] = v
            
            # Legacy copy of session and cache settings into app_conf
            if k.startswith('session_') or k.startswith('cache_'):
                self.app_conf[k] = v
            
        
        if 'session_data_dir' not in app_conf:
            app_conf['session_data_dir'] = os.path.join(app_conf['cache_dir'], 
                'sessions')
        if 'cache_data_dir' not in app_conf:
            app_conf['cache_data_dir'] = os.path.join(app_conf['cache_dir'], 
            'cache')

        # Setup the main template options dict
        self.template_options.update(myghty_template_options)
        
        # Setup several defaults for various template languages
        defaults = {}
        
        # Rearrange template options as default for Mako
        defaults['mako.directories'] = self.paths['templates']
        defaults['mako.filesystem_checks'] = True
        defaults['mako.output_encoding'] = self.default_charset
        if 'cache_dir' in app_conf:
            defaults['mako.module_directory'] = os.path.join(app_conf['cache_dir'], 
                                                     'templates')
        
        # Setup kid defaults
        defaults['kid.assume_encoding'] = 'utf-8'
        defaults['kid.encoding'] = self.default_charset
        
        # Merge template options into defaults
        defaults.update(self.template_options)
        self.template_options = defaults

        # Prepare our default template engine
        if template_engine == 'pylonsmyghty':
            self.add_template_engine('pylonsmyghty', None, myghty_template_options)
        elif template_engine == 'mako':
            self.add_template_engine('mako', '')
        elif template_engine in ['genshi', 'kid']:
            self.add_template_engine(template_engine, package + '.templates')
        
        # Save our errorware values
        self.errorware = errorware



"""REST decorators"""

import pylons
from pylons.decorator import decorator

def restrict(*methods):
    """Restricts access to the function depending on HTTP method
    
    Example:
    
    .. code-block:: Python
        
        class SomeController(BaseController):
            
            @pylons.rest.restrict('GET')
            def comment(self, id):
    """
    def check_methods(func, *args, **kwargs):
        """Wrapper for restrict"""
        if pylons.request.method not in methods:
            response = pylons.Response()
            response.headers['Allow'] = ','.join(methods)
            response.status_code = 405
            return response
        return func(*args, **kwargs)
    return decorator(check_methods)

def dispatch_on(**method_map):
    """Dispatches to alternate controller methods based on HTTP method
    
    Multiple keyword arguments should be passed, with the keyword corresponding
    to the HTTP method to dispatch on (DELETE, POST, GET, etc.) and the
    value being the function to call. The value should be a string indicating
    the name of the function to dispatch to.
    
    Example:
    
    .. code-block:: Python
    
        class SomeController(BaseController):
            
            @pylons.rest.dispatch_on(POST='create_comment')
            def comment(self):
                # Do something with the comment
            
            def create_comment(self, id):
                # Do something if its a post to comment
    """
    def dispatcher(func, self, *args, **kwargs):
        """Wrapper for dispatch_on"""
        alt_method = method_map.get(pylons.request.method)
        if alt_method:
            alt_method = getattr(self, alt_method)
            return self._inspect_call(alt_method, **kwargs)
        return func(self, *args, **kwargs)
    return decorator(dispatcher)

__all__ = ['restrict', 'dispatch_on']



"""Base objects to be exported for use in Controllers"""
from paste.registry import StackedObjectProxy
from paste.wsgiwrappers import WSGIResponse as Response

from pylons.controllers import Controller, XMLRPCController
from pylons.decorators import jsonify

c = StackedObjectProxy(name="C")
g = StackedObjectProxy(name="G")
cache = StackedObjectProxy(name="Cache")
session = StackedObjectProxy(name="Session")
request = StackedObjectProxy(name="Request")
buffet = StackedObjectProxy(name="Buffet")
h = StackedObjectProxy(name="h")

translator = StackedObjectProxy(name="Translator")

__all__ = ['Controller', 'RPCController', 'Response', 'jsonify']



"""Paste Template and Pylons utility functions

PylonsTemplate is a Paste Template sub-class that configures the source
directory and default plug-ins for a new Pylons project. The minimal template
provides a more minimal template with less additional directories and layout.
"""
import warnings

from paste.script.templates import Template

import pylons
import pylons.helpers
import pylons.i18n

def func_move(name, moved_to='pylons.i18n'):
    return ("The %s function has moved to %s, please update your import "
            "statements to reflect the move." % (name, moved_to))

def deprecated(func, message):
    def deprecated_method(*args, **kargs):        
        warnings.warn(message, DeprecationWarning, 2)
        return func(*args, **kargs)
    deprecated_method.__doc__ = message + "\n\n" + func.__doc__
    return deprecated_method

get_lang = deprecated(pylons.i18n.get_lang, func_move('get_lang'))
set_lang = deprecated(pylons.i18n.set_lang, func_move('set_lang'))
_ = deprecated(pylons.i18n._, func_move('_'))
log = deprecated(pylons.helpers.log, func_move('log', moved_to='pylons.helpers'))
    
def get_prefix(environ):
    if 'paste.config' in environ:
        prefix = environ['paste.config']['app_conf'].get('prefix', '')
    else:
        # Not ideal but if the error occurs before the paste.config is available not a lot we can do
        prefix = ''
    if not prefix:
        if environ.get('SCRIPT_NAME', '') != '':
            prefix = environ['SCRIPT_NAME']
    return prefix

def class_name_from_module_name(module_name):
    """Takes a module name and returns the name of the class it defines.

    If the module name contains dashes, they are replaced with underscores.
    
    Example::
    
        >>> class_name_from_module_name('with-dashes')
        'WithDashes'
        >>> class_name_from_module_name('with_underscores')
        'WithUnderscores'
        >>> class_name_from_module_name('oneword')
        'Oneword'
    """
    words = module_name.replace('-', '_').split('_')
    return ''.join([w.title() for w in words])


def config_get(key, default=None):
    """Return a value from ``CONFIG``. Supports both ``paste.config.CONFIG``
    and the older ``paste.deploy.CONFIG``.

    This is for Pylons internal use only; to support both versions of the
    ``CONFIG`` objects, depending on which is used by the current Pylons
    web app."""
    try:
        # environ['paste.config'] is assured to be the correct CONFIG
        # during requests
        CONFIG = pylons.request.environ['paste.config']
        value = CONFIG['app_conf'].get(key, default)
    except TypeError:
        # TypeError: pylons.request isn't registered (this function call was
        # made outside of a web request). fall back to paste.config.CONFIG
        # directly, and finally paste.deploy.CONFIG
        try:
            from paste.config import CONFIG
            value = CONFIG['app_conf'].get(key, default)
        except AttributeError:
            from paste.deploy import CONFIG
            value = CONFIG['app_conf'].get(key, default)
    return value

class ContextObj(object):
    """ The 'c' object, with strict attribute access (raises an Exception when
    the attribute does not exist) """
    pass

class AttribSafeContextObj(object):
    """ The 'c' object, with lax attribute access (returns '' when the attribute
    does not exist) """
    def __getattr__(self, name):
        try:
            return object.__getattribute__(self, name)
        except AttributeError:
            return ''
    def __repr__(self):
        attrs = [(name, value)
                 for name, value in self.__dict__.items()
                 if not name.startswith('_')]
        attrs.sort()
        parts = []
        for name, value in attrs:
            value_repr = repr(value)
            if len(value_repr) > 70:
                value_repr = value_repr[:60] + '...' + value_repr[-5:]
            parts.append('%s=%s' % (name, value_repr))
        return '<%s.%s %s %s>' % (
            self.__class__.__module__,
            self.__class__.__name__,
            hex(id(self)).split('x')[1],
            ' '.join(parts))

class PylonsTemplate(Template):
    _template_dir = 'templates/default_project'
    summary = 'Pylons application template'
    egg_plugins = ['Pylons', 'WebHelpers']

class MinimalPylonsTemplate(Template):
    _template_dir = 'templates/minimal_project'
    summary = 'Pylons minimal application template'
    egg_plugins = ['Pylons', 'WebHelpers']
    

__all__ = ['AttribSafeContextObj', 'ContextObj', 'Helpers', 'class_name_from_module_name',
    'log', '_', 'set_lang', 'get_lang']
__pudge_all__ = __all__ + ['MinimalPylonsTemplate', 'PylonsTemplate']



"""Standard Controllers intended for sub-classing by web developers"""
import inspect
import sys
import types
import warnings
import xmlrpclib

from paste.httpexceptions import HTTPException
from paste.deploy.converters import asbool

import pylons
from pylons.helpers import abort

XMLRPC_MAPPING = {str:'string', list:'array', int:'int', bool:'boolean',
                  float:'double', dict:'struct', 
                  xmlrpclib.DateTime:'dateTime.iso8601',
                  xmlrpclib.Binary:'base64'}

def xmlrpc_sig(args):
    """Returns a list of the function signature in string format based on a 
    tuple provided by xmlrpclib."""
    signature = []
    for param in args:
        for type, xml_name in XMLRPC_MAPPING.iteritems():
            if isinstance(param, type):
                signature.append(xml_name)
                break
    return signature

def xmlrpc_fault(code, message):
    """Convienence method to return a Pylons response XMLRPC Fault"""
    fault = xmlrpclib.Fault(code, message)
    return pylons.Response(xmlrpclib.dumps(fault, methodresponse=True))

def trim(docstring):
    """Yanked from PEP 237, strips the whitespace from Python doc strings"""
    if not docstring:
        return ''
    # Convert tabs to spaces (following the normal Python rules)
    # and split into a list of lines:
    lines = docstring.expandtabs().splitlines()
    # Determine minimum indentation (first line doesn't count):
    indent = sys.maxint
    for line in lines[1:]:
        stripped = line.lstrip()
        if stripped:
            indent = min(indent, len(line) - len(stripped))
    # Remove indentation (first line is special):
    trimmed = [lines[0].strip()]
    if indent < sys.maxint:
        for line in lines[1:]:
            trimmed.append(line[indent:].rstrip())
    # Strip off trailing and leading blank lines:
    while trimmed and not trimmed[-1]:
        trimmed.pop()
    while trimmed and not trimmed[0]:
        trimmed.pop(0)
    # Return a single string:
    return '\n'.join(trimmed)

class Controller(object):
    """Standard Pylons Controller for Web Requests
    
    The Pylons Controller handles incoming web requests that are dispatched
    from the custom Myghty Routes resolver. These requests result in a new
    instance of the Controller being created, which is then called with the
    dict options from the Routes match.
    
    By default, the Controller will search and attempt to call a ``__before__``
    method before calling the action, and will try to call a ``__after__``
    method after the action was called. These two methods can act as filters
    controlling access to the action, setup variables/objects for use with a
    set of actions, etc.
    
    Each action to be called is inspected with ``_inspect_call`` so that it is
    only passed the arguments in the Routes match dict that it asks for. The only
    exception to the dict is that the Myghty ``ARGS`` variable is included.
    
    In the event that an action is not found to handle the request, the Controller
    will raise an "Action Not Found" error if in debug mode, otherwise a ``404 Not Found``
    error will be returned.
    """
    __pudge_all__ = ['_inspect_call', '__call__', '_attach_locals']
    
    def _attach_locals(self):
        """Attach Pylons special objects to the controller
        
        When debugging, the Pylons special objects are unavailable because they
        are thread locals. This function pulls the actual object and attaches it
        to the controller so that it can be examined for debugging purposes.

        Deprecated (Nov 30 2006); Pylons special objects are now available
        within the interactive debugger.
        """
        warnings.warn('_attach_locals is deprecated: Pylons special objects '
                      'are now available within the interactive debugger',
                      DeprecationWarning, 2)
        self.c = pylons.c._current_obj()
        self.g = pylons.g._current_obj()
        self.cache = pylons.cache._current_obj()
        self.session = pylons.session._current_obj()
        self.request = pylons.request._current_obj()
        self.buffet = pylons.buffet._current_obj()

    def _inspect_call(self, func, **kargs):
        """Calls a function with as many arguments from args and kargs as
        possible
        
        Given a function, inspect_call will inspect the function args and call
        it with no further keyword args than it asked for.
        
        If the function has been decorated, it is assumed that the decorator
        preserved the function signature.
        """
        argspec = inspect.getargspec(func)
        kargs = self._get_method_args()
        
        # Hide the traceback for everything above this controller
        __traceback_hide__ = 'before_and_this'
        
        c = pylons.c._current_obj()
        if argspec[2]:
            for k, val in kargs.iteritems():
                setattr(c, k, val)
            result = func(**kargs)
        else:
            args = []
            argnames = argspec[0][1:]
            for name in argnames:
                if name in kargs:
                    setattr(c, name, kargs[name])
                    args.append(kargs[name])
            result = func(*args)
        if isinstance(result, types.GeneratorType):
            return pylons.Response(result)
        else:
            return result

    def _get_method_args(self):
        """Retrieve the method arguments to use with inspect call
        
        By default, this uses Routes to retrieve the arguments, override
        this method to customize the arguments your controller actions are
        called with."""
        req = pylons.request._current_obj()
        kargs = req.environ['pylons.routes_dict'].copy()
        kargs.update(dict(environ=req.environ, 
                          start_response=self.start_response))
        return kargs
    
    def _dispatch_call(self):
        """Handles dispatching the request to the function using Routes"""
        req = pylons.request._current_obj()
        action = req.environ['pylons.routes_dict'].get('action')
        action_method = action.replace('-', '_')
        func = getattr(self, action_method, None)
        if isinstance(func, types.MethodType):
            response = self._inspect_call(func)
        else:
            if asbool(req.environ['paste.config']['global_conf'].get('debug')):
                raise NotImplementedError(
                    'Action %s is not implemented' % action)
            else:
                response = pylons.Response(code=404)
        return response
    
    def __call__(self, *args, **kargs):
        """Makes our controller a callable to handle requests
        
        This is called when dispatched to as the Controller class docs explain
        more fully.
        """
        req = pylons.request._current_obj()
        
        # Keep private methods private
        if req.environ['pylons.routes_dict'].get('action', '').startswith('_'):
            return pylons.Response(code=404)
        
        if hasattr(self, '__before__'):
            self._inspect_call(self.__before__, **kargs)

        if not hasattr(self, '__after__'):
            response = self._dispatch_call()
        else:
            try:
                response = self._dispatch_call()
            except HTTPException:
                self._inspect_call(self.__after__)
                raise
            self._inspect_call(self.__after__)
        return response

class WSGIController(Controller):
    """WSGI Controller that follows WSGI spec for calling and return values
    
    This function works identically to the normal Controller, however it is called
    with the WSGI interface, and behaves as a WSGI application calling start_response
    and returning an iterable as content.
    """
    def __call__(self, environ, start_response):
        self.start_response = start_response
        
        # Keep private methods private
        if environ['pylons.routes_dict'].get('action', '').startswith('_'):
            return pylons.Response(code=404)
        
        if hasattr(self, '__before__'):
            self._inspect_call(self.__before__)

        if not hasattr(self, '__after__'):
            response = self._dispatch_call()
        else:
            try:
                response = self._dispatch_call()
            except HTTPException:
                self._inspect_call(self.__after__)
                raise
            self._inspect_call(self.__after__)
        
        if hasattr(response, 'wsgi_response'):
            # Copy the response object into the testing vars if we're testing
            if 'paste.testing_variables' in environ:
                environ['paste.testing_variables']['response'] = response
            return response(environ, start_response)
        
        return response

class XMLRPCController(WSGIController):
    """XML-RPC Controller"""

    max_body_length = 4194304

    def _get_method_args(self):
        return self.rpc_kargs

    def __call__(self, environ, start_response):
        # Pull out the length, return an error if there is no valid
        # length or if the length is larger than the max_body_length.
        length = environ.get('CONTENT_LENGTH')
        if length:
            length = int(length)
        else:
            # No valid Content-Length header found
            abort(411)
        if length > self.max_body_length or length == 0:
            abort(413, "XML body too large")

        body = environ['wsgi.input'].read(int(environ['CONTENT_LENGTH']))
        rpc_args, orig_method = xmlrpclib.loads(body)
        
        method = self._find_method_name(orig_method)
        if not hasattr(self, method):
            return xmlrpc_fault(0, "No method by that name")

        func = getattr(self, method)

        # Signature checking for params
        if hasattr(func, 'signature'):
            valid_args = False
            params = xmlrpc_sig(rpc_args)
            for sig in func.signature:
                # Next sig if we don't have the same amount of args
                if len(sig)-1 != len(rpc_args):
                    continue

                # If the params match, we're valid
                if params == sig[1:]:
                    valid_args = True
                    break

            if not valid_args:
                return xmlrpc_fault(0, "Incorrect argument signature. %s recieved does "
                                    "not match %s signature for method %s" % \
                                    (params, func.signature, orig_method))

        # Change the arg list into a keyword dict based off the arg
        # names in the functions definition
        arglist = inspect.getargspec(func)[0][1:]
        kargs = dict(zip(arglist, rpc_args))
        kargs['action'], kargs['environ'] = method, environ
        kargs['start_response'] = start_response
        self.rpc_kargs = kargs
        self._func = func

        # Now that we know the method is valid, and the args are valid,
        # we can dispatch control to the default WSGIController
        return WSGIController.__call__(self, environ, start_response)

    def _dispatch_call(self):
        """Dispatch the call to the function chosen by __call__"""
        raw_response = self._inspect_call(self._func)
        if not isinstance(raw_response, xmlrpclib.Fault):
            raw_response = (raw_response,)

        response = xmlrpclib.dumps(raw_response, methodresponse=True)
        return pylons.Response(response)

    def _find_method_name(self, name):
        return name.replace('.', '_')

    def _publish_method_name(self, name):
        return name.replace('_', '.')

    def system_listMethods(self):
        """Returns a list of XML-RPC methods for this XML-RPC resource"""
        methods = []
        for method in dir(self):
            meth = getattr(self, method)

            # Only methods have this attribute
            if not method.startswith('_') and hasattr(meth, 'im_self'):
                methods.append(self._publish_method_name(method))
        return methods
    system_listMethods.signature = [ ['array'] ]

    def system_methodSignature(self, name):
        """Returns an array of array's for the valid signatures for a method.

        The first value of each array is the return value of the method. The
        result is an array to indicate multiple signatures a method may be
        capable of.
        """
        if hasattr(self, name):
            method = getattr(self, name)
            if hasattr(method, 'signature'):
                return getattr(method, 'signature')
            else:
                return ''
        else:
            return xmlrpclib.Fault(0, 'No such method name')
    system_methodSignature.signature = [ ['array', 'string'],
                                         ['string', 'string'] ]

    def system_methodHelp(self, name):
        """Returns the documentation for a method"""
        if hasattr(self, name):
            method = getattr(self, name)
            help = getattr(method, 'help', None) or method.__doc__
            help = trim(help)
            sig = getattr(method, 'signature', None)
            if sig:
                help += "\n\nMethod signature: %s" % sig
            return help
        return xmlrpclib.Fault(0, "No such method name")
    system_methodHelp.signature = [ ['string', 'string'] ]

    
__all__ = ['Controller', 'WSGIController', 'XMLRPCController']



"""WSGI App Creator

This module is responsible for creating the basic Pylons WSGI application.
It's generally assumed that it will be called by Paste, though any WSGI 
application server could create and call the WSGI app as well.
"""
import gettext
import sys
import inspect
import warnings

import paste.wsgiwrappers
import paste.httpexceptions as httpexceptions
import paste.registry
from paste.wsgiwrappers import WSGIRequest

from routes import request_config
from routes.middleware import RoutesMiddleware

import pylons
import pylons.legacy
import pylons.templating
from pylons.i18n import set_lang
from pylons.util import ContextObj, AttribSafeContextObj, class_name_from_module_name
from pylons.controllers import Controller, WSGIController

class PylonsBaseWSGIApp(object):
    """Basic Pylons WSGI Application

    This basic WSGI app is provided should a web developer want to
    get access to the most basic Pylons web application environment
    available. By itself, this Pylons web application does little more
    than dispatch to a controller and setup the context object, the
    request object, and the globals object.
    
    Additional functionality like sessions, and caching can be setup by
    altering the ``environ['pylons.environ_config']`` setting to indicate
    what key the ``session`` and ``cache`` functionality should come from.
    
    Resolving the URL and dispatching can be customized by sub-classing or
    "monkey-patching" this class. Subclassing is the preferred approach.
    """
    def __init__(self, package_name, globals, helpers=None):
        """Initialize a base Pylons WSGI application
        
        The base Pylons WSGI application requires several keywords, the package
        name, and the globals object. If no helpers object is provided then h
        will be None.
        """
        self.helpers = helpers
        self.globals = globals
        self.package_name = package_name
        config = globals.pylons_config
        self.settings = dict(content_type='text/html', 
                             charset=config.default_charset)
        
        # Create the redirect function we'll use and save it
        def redirect_to(url):
            raise httpexceptions.HTTPFound(url)
        self.redirect_to = redirect_to
        
        # Initialize Buffet and all our template engines, default engine is the
        # first in the template_engines list
        def_eng = config.template_engines[0]
        self.buffet = pylons.templating.Buffet(def_eng['engine'], 
            template_root=def_eng['template_root'], **def_eng['template_options'])
        for e in config.template_engines[1:]:
            self.buffet.prepare(e['engine'], template_root=e['template_root'], 
                alias=e['alias'], **e['template_options'])
    
    def __call__(self, environ, start_response):
        req = self.setup_app_env(environ, start_response)
        if 'paste.testing_variables' in environ:
            self.load_test_env(environ)
            if environ['PATH_INFO'] == '/_test_vars':
                paste.registry.restorer.save_registry_state(environ)
                start_response('200 OK', [('Content-type','text/plain')])
                return ['%s' % paste.registry.restorer.get_request_id(environ)]
        
        controller = self.resolve(environ, start_response)
        response = self.dispatch(controller, environ, start_response)
        
        if 'paste.testing_variables' in environ and hasattr(response, 'wsgi_response'):
            environ['paste.testing_variables']['response'] = response
        
        # Transform HttpResponse objects into WSGI response
        if hasattr(response, 'wsgi_response'):
            return response(environ, start_response)
        elif response:
            return response
        
        raise Exception, "No content returned by controller: %s" % controller.__name__
    
    def setup_app_env(self, environ, start_response):
        """Setup and register all the Pylons objects with the registry"""
        req = WSGIRequest(environ)
        
        registry = environ['paste.registry']
                
        # Setup the basic pylons global objects
        registry.register(paste.wsgiwrappers.settings, self.settings)
        registry.register(pylons.request, req)
        registry.register(pylons.buffet, self.buffet)
        registry.register(pylons.g, self.globals)
        registry.register(pylons.h, self.helpers or \
                          pylons.legacy.load_h(self.package_name))
        
        # Setup the translator global object
        registry.register(pylons.translator, gettext.NullTranslations())
        lang = self.globals.pylons_config.app_conf.get('lang')
        if lang:
            set_lang(lang)
        
        if self.globals.pylons_config.strict_c:
            registry.register(pylons.c, ContextObj())
        else:
            registry.register(pylons.c, AttribSafeContextObj())
        
        econf = environ['pylons.environ_config']
        if econf.get('session'):
            registry.register(pylons.session, environ[econf['session']])
        if econf.get('cache'):
            registry.register(pylons.cache, environ[econf['cache']])
        return req
    
    def resolve(self, environ, start_response):
        """Uses dispatching information found in 
        ``environ['wsgiorg.routing_args']`` to retrieve a controller name and
        return the controller instance from the appropriate controller 
        module"""
        # Update the Routes config object in case we're using Routes
        config = request_config()
        config.redirect = self.redirect_to
        match = environ['wsgiorg.routing_args'][1]
        
        environ['pylons.routes_dict'] = match
        controller = match.get('controller')
        if not controller:
            return None
        
        # Pull the controllers class name, import controller
        full_module_name = self.package_name + '.controllers.' \
            + controller.replace('/', '.')
        
        # Hide the traceback here if the import fails (bad syntax and such)
        __traceback_hide__ = 'before_and_this'
        
        __import__(full_module_name)
        module_name = controller.split('/')[-1]
        class_name = class_name_from_module_name(module_name) + 'Controller'
        return getattr(sys.modules[full_module_name], class_name)
        
    def dispatch(self, controller, environ, start_response):
        """Dispatches to a controller, will instantiate the controller if
        necessary"""
        if not controller:
            raise httpexceptions.HTTPNotFound()
        match = environ['pylons.routes_dict']
        
        # Older subclass of Controller
        if inspect.isclass(controller) and not issubclass(controller, WSGIController) and \
                issubclass(controller, Controller):
            controller = controller()
            controller.start_response = start_response
            
            return controller(**match)
                
        # If it's a class, instantiate it
        if not hasattr(controller, '__class__') or \
            getattr(controller, '__class__') == type:
            controller = controller()
        
        # Controller is assumed to handle a WSGI call
        return controller(environ, start_response)
    
    def load_test_env(self, environ):
        """Sets up our Paste testing environment"""
        testenv = environ['paste.testing_variables']
        testenv['req'] = pylons.request._current_obj()
        testenv['c'] = pylons.c._current_obj()
        testenv['g'] = pylons.g._current_obj()
        testenv['h'] = pylons.h._current_obj()
        testenv['pylons_config'] = self.globals.pylons_config
        econf = environ['pylons.environ_config']
        if econf.get('session'):
            testenv['session'] = environ[econf['session']]
        if econf.get('cache'):
            testenv['cache'] = environ[econf['cache']]
    
class PylonsApp(object):
    """Setup the Pylons default environment
    
    Pylons App sets up the basic Pylons app, and initializes the global
    object, sessions and caching. Sessions and caching can be overridden
    in the config object by supplying other keys to look for in the environ
    where objects for the session/cache will be. If they're set to none,
    then no session/cache objects will be available.
    """
    def __init__(self, config, default_charset=None, helpers=None, g=None,
                 use_routes=True):
        self.config = config
        if default_charset:
            warnings.warn(
                "The 'default_charset' keyword argument to the PylonsApp constructor is "
                "deprecated. Please specify 'default_charset' to the Config object in your "
                'config/environment.py file instead, e.g.:\n'
                'return pylons.config.Config(myghty, map, paths, '
                "default_charset='%s')" % default_charset, DeprecationWarning, 2)
            self.config.default_charset = default_charset

        if helpers is None or g is None:
            warnings.warn(
                'Pylons 0.9.3 and above now explicitly pass helpers and g '
                'references to the PylonsApp constructor. Please update your '
                'middleware.py with:\n\n'
                '    import MYPROJ.lib.app_globals as app_globals\n'
                '    import MYPROJ.lib.helpers\n\n'
                'Then edit the PylonsApp instantiation with:\n\n'
                '    app = pylons.wsgiapp.PylonsApp(config, '
                'helpers=MYPROJ.lib.helpers,\n'
                '                                   g=app_globals.Globals)\n\n'
                'where MYPROJ is the name of your project.\n',
                DeprecationWarning, 2)
            
        if not g:
            try:
                globals_package = __import__(config.package + '.lib.app_globals',
                                             globals(), locals(), ['Globals'])
                g = getattr(globals_package, 'Globals')
            except ImportError:
                pass
        # Assign a default globals object, and instantiate it
        if not g:
            g = type("Globals", (), {})()
        else:
            g = g(config.global_conf, config.app_conf, config=config)
        g.pylons_config = config
        
        # Create the base Pylons wsgi app
        app = PylonsBaseWSGIApp(config.package, g, helpers=helpers)
        if use_routes:
            app = RoutesMiddleware(app, config.map)
        
        # Pull user-specified environ overrides, or just setup default
        # session and caching objects
        self.econf = econf = config.environ_config.copy()
        if 'session' not in econf:
            from beaker.session import SessionMiddleware
            econf['session'] = 'beaker.session'
            app = SessionMiddleware(app, config.global_conf, 
                auto_register=True, **config.app_conf)
        
        if 'cache' not in econf:
            from beaker.cache import CacheMiddleware
            econf['cache'] = 'beaker.cache'
            app = CacheMiddleware(app, config.global_conf, **config.app_conf)
        
        self.globals = g
        self.app = app
    
    def __call__(self, environ, start_response):
        environ['pylons.environ_config'] = self.econf
        return self.app(environ, start_response)



"""Caching decorators"""
import inspect

from paste.deploy.converters import asbool

import pylons
from pylons.decorator import decorator

def beaker_cache(key="cache_default", expire="never", type="dbm", 
    query_args=False):
    """Cache decorator utilizing Beaker. Caches action or other function that
    returns a pickle-able object as a result.
    
    Optional arguments:
    
    key
        None - No variable key, uses function name as key
        "cache_default" - Uses all function arguments as the key
        string - Use kwargs[key] as key
        list - Joins the arguments in the list
    expire
        Time in seconds before cache expires, defaults to never
    type
        Type of cache to use: dbm, memory, file, memcached
    query_args
        Uses the query arguments as the key, defaults to False

    If cache_enabled is set to False in the .ini file, then cache is disabled 
    globally.
    """
    def wrapper(func, *args, **kwargs):
        """Decorator wrapper"""
        enabled = pylons.g.pylons_config.app_conf.get("cache_enabled", "True")
        if not asbool(enabled):
            return func(*args, **kwargs)
        
        my_cache = pylons.cache.get_cache(func.__module__ + "." + func.__name__)
        cache_key = _make_key(func, key, args, kwargs, query_args)
        
        if expire == "never":
            cache_expire = None
        else:
            cache_expire = expire
        
        content = my_cache.get_value(cache_key,
            createfunc=lambda: func(*args, **kwargs), type=type,
            expiretime=cache_expire)
        return content
    return decorator(wrapper)

def _make_key(func, key, args, kwargs, query_args):
    """Helps make unique key from largs, kewargs and request.GET"""
    if key == "cache_default":
        if query_args:
            cache_key = repr(dict(pylons.request.GET))
        else:
            cache_key = repr(kwargs.items())
            largs_keys = _make_dict_from_args(func, args)
            cache_key += repr(largs_keys.items())
    elif not key:
        cache_key = func.__name__
    else:
        if query_args:
            dic = pylons.request.GET
        else:
            largs_keys = _make_dict_from_args(func, args)
            dic = kwargs.copy()
            dic.update(largs_keys)
        if isinstance(key, list):
            cache_key = " ".join(["%s=%s" % (k, dic[k]) for k in key])
        else:
            cache_key = "%s=%s" % (key, dic[key])
    return cache_key

def _make_dict_from_args(func, args):
    """Inspects function for name of args"""
    args_keys = {}
    for i, arg in enumerate(inspect.getargspec(func)[0]):
        if arg != "self":
            args_keys[arg] = args[i]
    return args_keys



"""Pylons Decorators: ``jsonify``, ``validate``, REST, and Cache decorators"""
import simplejson as json
import formencode.api as api
from formencode import htmlfill
import formencode.variabledecode as variabledecode
import pylons
from pylons.decorator import decorator

def jsonify(func, *args, **kwargs):
    """Action decorator that formats output for JSON
    
    Given a function that will return content, this decorator will
    turn the result into JSON, with a content-type of 'text/javascript'
    and output it.
    """
    response = pylons.Response()
    response.headers['Content-Type'] = 'text/javascript'
    response.content.append(json.dumps(func(*args, **kwargs)))
    return response
jsonify = decorator(jsonify)

def validate(schema=None, validators=None, form=None, variable_decode=False,
             dict_char='.', list_char='-', post_only=True):
    """Validate input either for a FormEncode schema, or individual validators
    
    Given a form schema or dict of validators, validate will attempt to validate
    the schema or validator list as long as a POST request is made. No 
    validation is performed on GET requests.
    
    If validation was succesfull, the valid result dict will be saved
    as ``self.form_result``. Otherwise, the action will be re-run as if it was a
    GET, and the output will be filled by FormEncode's htmlfill to fill in the
    form field errors.
    
    If you'd like validate to also check GET (query) variables during its 
    validation, set the ``post_only`` keyword argument to False.
    
    Example:
    
    .. code-block:: Python
        
        class SomeController(BaseController):
            
            def create(self, id):
                return render_response('/myform.myt')
            
            @validate(schema=model.forms.myshema(), form='create')
            def update(self, id):
                # Do something with self.form_result
                pass
    """
    def wrapper(func, self, *args, **kwargs):
        """Decorator Wrapper function"""
        errors = {}
        if not pylons.request.method == 'POST':
            return func(self, *args, **kwargs)
        if post_only:
            params = pylons.request.POST.copy()
        else:
            params = pylons.request.params.copy()
        if variable_decode:
            decoded = variabledecode.variable_decode(params, dict_char,
                                                     list_char)
        else:
            decoded = params

        if schema:
            try:
                self.form_result = schema.to_python(decoded)
            except api.Invalid, e:
                errors = e.unpack_errors(variable_decode, dict_char, list_char)
        if validators:
            if isinstance(validators, dict):
                if not hasattr(self, 'form_result'):
                    self.form_result = {}
                for field, validator in validators.iteritems():
                    try:
                        self.form_result[field] = \
                            validator.to_python(decoded[field] or None)
                    except api.Invalid, error:
                        errors[field] = error
        if errors:
            pylons.request.environ['REQUEST_METHOD'] = 'GET'
            pylons.request.environ['pylons.routes_dict']['action'] = form
            response = self._dispatch_call()
            form_content = "".join(response.content)
            response.content = [htmlfill.render(form_content, params, errors)]
            return response
        return func(self, *args, **kwargs)
    return decorator(wrapper)

__all__ = ['jsonify', 'validate']
